# user_manager.py
import uuid
from typing import Optional

from fastapi import Depends, Request
from fastapi_users import BaseUserManager, UUIDIDMixin
# from fastapi_users_sqlalchemy import SQLAlchemyUserDatabase
from fastapi_users.db import SQLAlchemyUserDatabase
from .database import get_async_session

from fastapi import Depends, Request, HTTPException, status # <-- 确保 HTTPException 在这里
from sqlalchemy.exc import IntegrityError # <-- 确保这一行存在

from umanager.server.models.models import User, UserCreate # <--- 在这里添加 UserCreate 的导入

from passlib.context import CryptContext # 确保这里引入了 CryptContext

# ---------- 确保 pwd_context 在模块级别定义 ----------
# 它应该在任何使用它的类或函数之外
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")

# 替换为您的安全密钥，最好从环境变量中获取
SECRET = "YOUR_SUPER_SECRET_KEY_REPLACE_ME" # !!! 重要：在生产环境中，请务必使用环境变量


class UserManager(UUIDIDMixin, BaseUserManager[User, uuid.UUID]):
    # ... (reset_password_token_secret, verification_token_secret 保持不变)

    async def create(
        self,
        user_create: UserCreate,
        safe: bool = False,
        request: Optional[Request] = None,
    ) -> User:
        existing_user = await self.user_db.get_by_email(user_create.email)
        if existing_user is not None:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="A user with this email already exists."
            )

        hashed_password = pwd_context.hash(user_create.password)

        # 使用 model_dump 获取 Pydantic V2 的字典表示
        user_data = user_create.model_dump(exclude_unset=True)
        user_data["hashed_password"] = hashed_password
        user_data["id"] = uuid.uuid4()

        if "password" in user_data: # 确保移除明文密码
            del user_data["password"]

        # 创建 SQLAlchemy User 实例
        # 这里的 self.user_db.user_table 就是 models.User
        new_user = self.user_db.user_table(**user_data) 
        
        # 确保默认字段被设置
        new_user.is_active = True
        new_user.is_superuser = False
        new_user.is_verified = False

        # created_user = await self.user_db.create(new_user)
        try:
            self.user_db.session.add(new_user)
            await self.user_db.session.commit()
            await self.user_db.session.refresh(new_user) # 刷新以获取可能由数据库生成的字段（如默认值）
        except IntegrityError:
            await self.user_db.session.rollback()
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="A database integrity error occurred (e.g., duplicate email)."
            )
        # await self.on_after_register(created_user, request)
        # return created_user
        await self.on_after_register(new_user, request) # 这里也是 new_user
        return new_user

async def get_user_db(session = Depends(get_async_session)):
    yield SQLAlchemyUserDatabase(session, User)



async def get_user_manager(user_db: SQLAlchemyUserDatabase = Depends(get_user_db)):
    yield UserManager(user_db)


# uv venv .venv --python 3.11 
# python -c "import pydantic; print(pydantic.VERSION)"



import uuid
from typing import Optional

from sqlalchemy import String, Boolean, Column
from sqlalchemy.dialects.mysql import CHAR # Use CHAR for UUID to store as fixed length string
from fastapi_users.db import SQLAlchemyBaseUserTableUUID
from pydantic import BaseModel, Field

from umanager.server.utils import Base # 引入Base

# SQLAlchemy 用户模型
# 使用 SQLAlchemyBaseUserTableUUID 来自动处理 UUID 主键
class User(SQLAlchemyBaseUserTableUUID, Base):
    __tablename__ = "users" # 表名
    
    # 可以在这里添加额外的用户属性
    first_name: str = Column(String(255), nullable=True)
    last_name: str = Column(String(255), nullable=True)
    is_admin: bool = Column(Boolean, default=False)

# Pydantic 模型 (用于请求和响应)
class UserRead(BaseModel):
    id: uuid.UUID
    email: str
    is_active: bool = True
    is_superuser: bool = False
    is_verified: bool = False
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    is_admin: bool = False

    class Config:
        from_attributes = True # updated from orm_mode = True

class UserCreate(BaseModel):
    email: str
    password: str
    first_name: Optional[str] = None
    last_name: Optional[str] = None

class UserUpdate(UserCreate):
    is_active: Optional[bool] = None
    is_superuser: Optional[bool] = None
    is_verified: Optional[bool] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    is_admin: Optional[bool] = None




from mcp.server.fastmcp import Context, FastMCP


# region MCP Math
mcp = FastMCP("resource")


@mcp.resource("config://settings")
def get_settings() -> str:
    """Get application settings."""
    return """{
  "theme": "dark",
  "language": "en",
  "debug": false
}"""

@mcp.resource("file://documents/{name}")
def read_document(name: str) -> str:
    """Read a document by name."""
    # This would normally read from disk
    path = "/Users/zhaoxuefeng/GitHub/prompt_writing_assistant/tests/temp_file/"
    path += name
    with open(path,'r') as f:
        return f.read()

@mcp.resource("file://notes/{name}")
def read_notes(name:str)-> str:
    """ 阅读备忘录"""

    return "你要做王子"

if __name__ == "__main__":
    mcp.run(transport="streamable-http")




# server
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from dotenv import load_dotenv, find_dotenv
from contextlib import asynccontextmanager, AsyncExitStack
from fastapi import FastAPI, Request
from fastapi.routing import APIRoute
import argparse
import uvicorn

from .mcp.math import mcp as fm_math
from .mcp.weather import mcp as fm_weather


default = 8007

dotenv_path = find_dotenv()
load_dotenv(dotenv_path, override=True)


# Combine both lifespans
@asynccontextmanager
async def combined_lifespan(app: FastAPI):
    # Run both lifespans
    async with AsyncExitStack() as stack:
        await stack.enter_async_context(fm_math.session_manager.run())
        await stack.enter_async_context(fm_weather.session_manager.run())
        yield

app = FastAPI(
    title="LLM Service",
    description="Provides an OpenAI-compatible API for custom large language models.",
    version="1.0.1",
    # debug=True, 
    # docs_url="/api-docs",
    lifespan=combined_lifespan
)

# --- Configure CORS ---
origins = [
    "*", 
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,  # Specifies the allowed origins
    allow_credentials=True,  # Allows cookies/authorization headers
    allow_methods=["*"],  # Allows all methods (GET, POST, OPTIONS, etc.)
    allow_headers=["*"],  # Allows all headers (Content-Type, Authorization, etc.)
)
# --- End CORS Configuration ---




app.mount("/math", fm_math.streamable_http_app()) # /math/mcp
app.mount("/weather", fm_weather.streamable_http_app()) # /weather/mcp





@app.get("/")
async def root():
    """server run"""
    return {"message": "LLM Service is running."}


@app.get("/api/status")
def status():
    return {"status": "ok"}

@app.get("/api/list-routes/")
async def list_fastapi_routes(request: Request):
    routes_data = []
    for route in request.app.routes:
        if isinstance(route, APIRoute):
            routes_data.append({
                "path": route.path,
                "name": route.name,
                "methods": list(route.methods),
                "endpoint": route.endpoint.__name__ # Get the name of the function
            })
    return {"routes": routes_data}



if __name__ == "__main__":
    # 这是一个标准的 Python 入口点惯用法
    # 当脚本直接运行时 (__name__ == "__main__")，这里的代码会被执行
    # 当通过 python -m YourPackageName 执行 __main__.py 时，__name__ 也是 "__main__"
    # 27
    
    parser = argparse.ArgumentParser(
        description="Start a simple HTTP server similar to http.server."
    )
    parser.add_argument(
        "port",
        metavar="PORT",
        type=int,
        nargs="?",  # 端口是可选的
        default=default,
        help=f"Specify alternate port [default: {default}]",
    )
    # 创建一个互斥组用于环境选择
    group = parser.add_mutually_exclusive_group()

    # 添加 --dev 选项
    group.add_argument(
        "--dev",
        action="store_true",  # 当存在 --dev 时，该值为 True
        help="Run in development mode (default).",
    )

    # 添加 --prod 选项
    group.add_argument(
        "--prod",
        action="store_true",  # 当存在 --prod 时，该值为 True
        help="Run in production mode.",
    )
    args = parser.parse_args()

    if args.prod:
        env = "prod"
    else:
        # 如果 --prod 不存在，默认就是 dev
        env = "dev"

    port = args.port

    if env == "dev":
        port += 100
        reload = True
        app_import_string = (
            f"{__package__}.__main__:app"  # <--- 关键修改：传递导入字符串
        )
    elif env == "prod":
        reload = False
        app_import_string = app
    else:
        reload = False
        app_import_string = app

    # 使用 uvicorn.run() 来启动服务器
    # 参数对应于命令行选项
    uvicorn.run(
        app_import_string, host="0.0.0.0", port=port, reload=reload  # 启用热重载
    )



from datetime import datetime
from mcp.server.fastmcp import FastMCP


def create_mcp():
    # region MCP Weather
    mcp = FastMCP("Weather")

    @mcp.tool()
    def get_weather(location: str) -> str:
        return "Cloudy"

    @mcp.tool()
    def get_time() -> str:
        return datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    # endregion
    return mcp



if __name__ == "__main__":
    mcp = create_mcp()
    mcp.run(transport="streamable-http")






from mcp.server.fastmcp import Context, FastMCP, Icon
from prompt_writing_assistant.file_manager import ContentManager, TextType
from mcp.server.fastmcp import FastMCP
from mcp.server.fastmcp.prompts import base
from pydantic import BaseModel, Field
from prompt_writing_assistant.prompt_helper import IntellectType,Intel


content_manager = ContentManager()
intels = Intel()

mcp = FastMCP("Content")

@mcp.prompt(title="Code Review")
def review_code(code: str) -> str:
    return f"Please review this code:\n\n{code}"


@mcp.prompt(title="Debug Assistant")
def debug_error(error: str) -> list[base.Message]:
    return [
        base.UserMessage("I'm seeing this error:"),
        base.UserMessage(error),
        base.AssistantMessage("I'll help debug that. What have you tried so far?"),
    ]


@mcp.tool()
def save_content(text:str)->str:
    """
    text : 需要存储的内容
    return : 返回是否存储成功的信息
    """
    result =content_manager.save_content_auto(
                text = text)
    return result

@mcp.tool()
def similarit_content(text: str,limit: int):
    """
    text : 待查询的文字, 进行匹配
    limit : 查询的数量
    """
    
    result = content_manager.similarity(
        content = text,
        limit = limit
    )
    return result

@mcp.tool()
def get_prompts(prompt_id: str,version: str = "1.0"):
    """
    prompt_id : 待查询的文字, 进行匹配
    limit : 查询的数量
    """
    result = intels.get_prompts_from_sql(
        table_name = "prompts_data",
        prompt_id = prompt_id,
        version=version,
        )
    return result

@mcp.tool()
def list_cities() -> list[str]:
    """Get a list of cities"""
    return ["London", "Paris", "Tokyo"]






class WeatherData(BaseModel):
    """Weather information structure."""

    temperature: float = Field(description="Temperature in Celsius")
    humidity: float = Field(description="Humidity percentage")
    condition: str
    wind_speed: float


@mcp.tool() # icons=[icon]
def get_weather(city: str) -> WeatherData:
    """Get weather for a city - returns structured data."""
    # Simulated weather data
    return WeatherData(
        temperature=22.5,
        humidity=45.0,
        condition="sunny",
        wind_speed=5.2,
    )


from PIL import Image as PILImage
from mcp.server.fastmcp import FastMCP, Image

@mcp.tool()
def create_thumbnail(image_path: str) -> Image:
    """Create a thumbnail from an image"""
    img = PILImage.open(image_path)
    img.thumbnail((100, 100))
    return Image(data=img.tobytes(), format="png")



if __name__ == "__main__":
    mcp.run(transport="streamable-http")
    # mcp.run(transport="sse")
    # search_mcp.run(transport="sse", mount_path="/search")





from dotenv import load_dotenv, find_dotenv

dotenv_path = find_dotenv()
load_dotenv(".env", override=True)

from .log import Log
import logging
Log_ = Log(console_level = logging.INFO, # 显示控制台的等级
             log_file_name="app.log")
logger = Log_.logger



"""
FastMCP quickstart example.

"""

from mcp.server.fastmcp import FastMCP
from prompt_writing_assistant.file_manager import ContentManager, TextType

# uv run mcp dev src/obsidian_sdk/mcp.py
# Create an MCP server
mcp = FastMCP("Demo")
content_manager = ContentManager()

@mcp.tool()
def save_content(text:str)->str:
    """
    text : 需要存储的内容

    return : 返回是否存储成功的信息
    """
    result =content_manager.save_content_auto(
                text = text)
    return result


@mcp.tool()
def similarit_content(text: str,limit: int):
    """
    text : 待查询的文字, 进行匹配
    limit : 查询的数量
    """
    
    result = content_manager.similarity(
        content = text,
        limit = limit
    )
    return result



@mcp.resource("file://notes/{name}")
def read_notes(name:str)-> str:
    """ 阅读备忘录"""

    return "你要做王子"


if __name__ == "__main__":
    # mcp.run(transport="streamable-http")
    mcp.run(transport="sse")
    # search_mcp.run(transport="sse", mount_path="/search")


# auth_backends.py
from fastapi_users.authentication import BearerTransport, JWTStrategy, AuthenticationBackend

from .user_manager import SECRET # 从 user_manager 引入 SECRET

bearer_transport = BearerTransport(tokenUrl="auth/jwt/login")

def get_jwt_strategy() -> JWTStrategy:
    return JWTStrategy(secret=SECRET, lifetime_seconds=3600)  # JWT有效期1小时

auth_backend = AuthenticationBackend(
    name="jwt",
    transport=bearer_transport,
    get_strategy=get_jwt_strategy,
)



import logging
import os
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler
import inspect
import json
import functools
import asyncio

# 定义自定义日志级别及其对应的整数值
logging.addLevelName(25, "NOTICE")
NOTICE = 25

# 创建一个函数，用于方便地调用自定义日志级别
def notice(self, msg, *args, **kws):
    if self.isEnabledFor(NOTICE):
        self._log(NOTICE, msg, args, **kws)

logging.Logger.notice = notice

class Log:
    _instance = None
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, console_level = logging.INFO, log_file_name="app.log"):
        self.Console_LOG_LEVEL = console_level
        self.log_file_name = log_file_name
        self.LOG_FILE_PATH = os.path.join("logs", log_file_name)
        os.makedirs(os.path.dirname(self.LOG_FILE_PATH), exist_ok=True)
        self.logger = self.get_logger()

    def get_logger(self):
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        if not logger.handlers:
            # --- 4. 配置 Formatter (格式化器) ---
            # 以后有一个标准化的日志要使用logger 而非标的则使用super-log
            formatter = logging.Formatter(
                "%(asctime)s $ %(created)f $ %(levelname)s $ %(funcName)s $ :%(lineno)d $ %(pathname)s $ %(message)s||"
            )
            # --- 5. 配置 Handler (处理器) ---

            # 5.1 控制台处理器 (StreamHandler)
            console_handler = logging.StreamHandler()
            console_handler.setLevel(self.Console_LOG_LEVEL)  # 控制台只显示 INFO 及以上级别的日志
            console_handler.setFormatter(formatter)
            logger.addHandler(console_handler)

            # 文件系统
            ## 主日志本
            file_handler = RotatingFileHandler(  # RotatingFileHandler: 按文件大小轮转
                self.LOG_FILE_PATH,
                maxBytes=20 * 1024 * 1024,  # 10 MB # maxBytes: 单个日志文件的最大字节数 (例如 10MB)
                backupCount=10, # backupCount: 保留的旧日志文件数量
                encoding="utf-8",
            )
            file_handler.setLevel(logging.DEBUG)  # 记录所有日志
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)
            
            ## 运行日志本
            file_handler_info = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_info.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_info.setLevel(logging.INFO)  # 记录本系统的日志
            file_handler_info.setFormatter(formatter)
            logger.addHandler(file_handler_info)


            ## 运行日志本
            file_handler_info = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_notice.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_info.setLevel(25)  # 记录本系统的日志
            file_handler_info.setFormatter(formatter)
            logger.addHandler(file_handler_info)

            ## 错误日志本
            file_handler_warning = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_error.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_warning.setLevel(logging.WARNING)  # 记录警告和错误
            file_handler_warning.setFormatter(formatter)
            logger.addHandler(file_handler_warning)

            ## 指定日志本 
            file_handler_super = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_caitical.log'),
                maxBytes=5 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_super.setLevel(logging.CRITICAL)  # 记录重点跟踪日志
            file_handler_super.setFormatter(formatter)
            logger.addHandler(file_handler_super)

        return logger


def log_func(logger):
    def outer_packing(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            kwargs_dict = json.dumps(kwargs,ensure_ascii=False)
            try:
                if asyncio.iscoroutinefunction(func):
                    # 如果被装饰的函数是协程，则 await 它
                    result = await func(*args, **kwargs)
                else:
                    # 否则，直接调用（同步函数）
                    result = func(*args, **kwargs)
                logger.notice(f'{func.__name__} & {kwargs_dict} & {result}')
            except Exception as e:
                logger.error(f'{func.__name__} & {kwargs_dict}  & {e}')
                raise 
            return result
        return wrapper
    return outer_packing



from mcp.server.fastmcp import FastMCP


# region MCP Math
mcp = FastMCP("Math")

@mcp.tool(description="A simple add tool")
def add(a: int, b: int) -> int:
    """Adds two integers.
       Args:
         a: The first integer.
         b: The second integer.
    """
    return a + b


@mcp.tool()
def multiply(a: int, b: int) -> int:
    """Multiply two integers.
       Args:
         a: The first integer.
         b: The second integer.
    """
    return a * b
# endregion


if __name__ == "__main__":
    mcp.run(transport="streamable-http")
    # mcp.run(transport="sse")
    # search_mcp.run(transport="sse", mount_path="/search")






# app/routers/admin.py
from fastapi import APIRouter, Depends, HTTPException, status
from fastapi import FastAPI, Header


# 模拟获取当前用户的依赖项
async def get_current_user(x_token: str = Header(...)):
    if x_token not in ["1234",
                       "5678"]:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="Invalid X-Token header")
    return {"username": "admin", "roles": ["user", "admin"]}


router = APIRouter(
    tags=["users"],
    dependencies=[Depends(get_current_user)] # 统一的依赖项列表
)


@router.get("/dashboard")
async def get_user(user_info :dict =  Depends(get_current_user)):
    pass
    return {"message": "Welcome to the admin dashboard!"}



# database.py
from typing import AsyncGenerator

from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker, DeclarativeBase

# 您的 MySQL 连接字符串
# 格式：mysql+aiomysql://user:password@host:port/database
# 注意：fastapi_users 的 sqlalchemy 适配器通常需要异步驱动，
# 所以我们使用 `aiomysql` (虽然你安装的是 mysql-connector-python，
# 但为了异步，我们实际会用 aiomysql，请确保安装它：pip install aiomysql)
DATABASE_URL = "mysql+aiomysql://root:1234@localhost:3306/prompts"

class Base(DeclarativeBase):
    pass

engine = create_async_engine(DATABASE_URL)
async_session_maker = sessionmaker(engine, class_=AsyncSession, expire_on_commit=False)

async def get_async_session() -> AsyncGenerator[AsyncSession, None]:
    async with async_session_maker() as session:
        yield session

# 创建所有定义的表（在应用启动时调用）
async def create_db_and_tables():
    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)




# app/routers/admin.py
from fastapi import APIRouter, Depends, HTTPException, status
from fastapi import FastAPI, HTTPException, Header


# 模拟获取当前用户的依赖项
async def get_current_user(x_token: str = Header(...)):
    if x_token != "valid-secret-token":
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="Invalid X-Token header")
    return {"username": "admin", "roles": ["user", "admin"]}

# 模拟一个管理员权限检查的依赖项
async def verify_admin_role(user: dict = Depends(get_current_user)):
    if "admin" not in user.get("roles", []):
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN, detail="Not enough permissions")
    return user


router = APIRouter(
    tags=["Admin"],
    dependencies=[Depends(get_current_user), Depends(verify_admin_role)] # 统一的依赖项列表
)



@router.get("/dashboard/")
async def get_admin_dashboard_data(user):
    # 这个函数会自动接收到 get_current_user 和 verify_admin_role 的结果（如果需要）
    # 但我们在这里不需要显式接收它们，因为它们只是做权限检查
    print(user,'user')
    return {"message": "Welcome to the admin dashboard!"}

@router.post("/settings/")
async def update_admin_settings(settings: dict):
    # 这个函数也会自动执行 get_current_user 和 verify_admin_role
    return {"message": "Admin settings updated", "settings": settings}

# 这个API也需要认证和管理员权限
@router.delete("/users/{user_id}")
async def delete_user(user_id: int):
    return {"message": f"User {user_id} deleted by admin."}
